# coding=utf-8

#    Copyright (C) 2008-2010  Luis Falcon

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.


import time
from mx import DateTime
import datetime
from osv import fields, osv
from tools.translate import _


class insurance (osv.osv):
	_name = "medical.insurance"
	_columns = {
		'name' : fields.char ('Number', size=64),
		'partner_id' : fields.many2one ('res.partner','Owner'),
		'company' : fields.many2one ('res.partner','Insurance Company'),
		'member_since' : fields.date ('Member since'),
		'member_exp' : fields.date ('Expiration date'),
		'category' : fields.char ('Category', size=64, help="Insurance company plan / category"),
		'type' : fields.selection([
                                ('state','State'),
                                ('labour_union','Labour Union / Syndical'),
                                ('private','Private'),

                                ], 'Insurance Type', select=True),

		'notes' : fields.text ('Extra Info'),

		}
insurance ()



class partner_patient (osv.osv):
	_name = "res.partner"
	_inherit = "res.partner"
	_columns = {
		'date' : fields.date('Partner since',help="Date of activation of the partner or patient"),
		'alias' : fields.char('alias', size=64),
		'ref': fields.char('ID Number', size=64, required=True),
                'is_patient' : fields.boolean('Patient', help="Check if the partner is a patient"),
                'is_doctor' : fields.boolean('Doctor', help="Check if the partner is a doctor"),
		'is_institution' : fields.boolean ('Institution', help="Check if the partner is a Medical Center"),
		'lastname' : fields.char('Last Name', size=128, help="Last Name"),
		'insurance' : fields.one2many ('medical.insurance','partner_id',"Insurance"),	
	}
        _sql_constraints = [
                ('ref_uniq', 'unique (ref)', 'The partner or patient code must be unique')
 		]

partner_patient ()


class product_medical (osv.osv):
	_name = "product.product"
	_inherit = "product.product"
	_columns = {
                'is_medicament' : fields.boolean('Medicament', help="Check if the product is a medicament"),
                'is_vaccine' : fields.boolean('Vaccine', help="Check if the product is a vaccine"),
                'is_bed' : fields.boolean('Ward', help="Check if the product is a bed on the medical center"),

	}
product_medical ()


#Add the partner relationship field to the contacts.
class partner_patient_address (osv.osv):
	_name = "res.partner.address"
	_inherit = "res.partner.address"
	_columns = {
		'relationship' : fields.char('Relationship', size=64),
		'relative_id' : fields.many2one('res.partner','Patient', domain=[('is_patient', '=', True)], help="Patient ID"),
	}
partner_patient_address ()

class procedure_code (osv.osv):
	_description = "Medical Procedure"
	_name = "medical.procedure"
	_columns = {
		'name': fields.char ('Code', size=16),
		'description' : fields.char ('Long Text', size=256),
		}

	def name_search(self, cr, uid, name, args=[], operator='ilike', context={}, limit=80):
        	args2 = args[:]
        	if name:
            		args += [('name', operator, name)]
            		args2 += [('description', operator, name)]
        	ids = self.search(cr, uid, args, limit=limit)
        	ids += self.search(cr, uid, args2, limit=limit)
        	res = self.name_get(cr, uid, ids, context)
        	return res

procedure_code ()



class pathology_category(osv.osv):
        def name_get(self, cr, uid, ids, context={}):
                if not len(ids):
                        return []
                reads = self.read(cr, uid, ids, ['name','parent_id'], context)
                res = []
                for record in reads:
                        name = record['name']
                        if record['parent_id']:
                                name = record['parent_id'][1]+' / '+name
                        res.append((record['id'], name))
                return res

        def _name_get_fnc(self, cr, uid, ids, prop, foo, faa):
                res = self.name_get(cr, uid, ids)
                return dict(res)
        def _check_recursion(self, cr, uid, ids):
                level = 100
                while len(ids):
                        cr.execute('select distinct parent_id from medical_pathology_category where id in ('+','.join(map(str,ids))+')')
                        ids = filter(None, map(lambda x:x[0], cr.fetchall()))
                        if not level:
                                return False
                        level -= 1
                return True

        _description='Pathology Categories'
        _name = 'medical.pathology.category'
        _columns = {
                'name': fields.char('Category Name', required=True, size=128),
                'parent_id': fields.many2one('medical.pathology.category', 'Parent Category', select=True),
                'complete_name': fields.function(_name_get_fnc, method=True, type="char", string='Name'),
                'child_ids': fields.one2many('medical.pathology.category', 'parent_id', 'Children Category'),
                'active' : fields.boolean('Active'),
        }
        _constraints = [
                (_check_recursion, 'Error ! You can not create recursive categories.', ['parent_id'])
        ]
        _defaults = {
                'active' : lambda *a: 1,
        }
        _order = 'parent_id,id'

pathology_category()


class pathology (osv.osv):
	_name = "medical.pathology"
	_description = "Pathologies"
	_columns = {
		'name' : fields.char ('Name', size=128, help="Pathology / disease name"),
		'code' : fields.char ('Code', size=5, help="Specific Code for the disease (eg, ICD-10, SNOMED...)"),
		'category' : fields.many2one('medical.pathology.category','Pathology Category'),
		'chromosome' : fields.char ('Affected Chromosome', size=128, help="chromosome number"),
		'protein' : fields.char ('Protein involved', size=128, help="Name of the protein(s) affected"),
		'gene' : fields.char ('Gene', size=128, help="Name of the gene(s) affected"),
		'info' : fields.text ('Extra Info'),
	}

        _sql_constraints = [
                ('code_uniq', 'unique (code)', 'The pathology code must be unique')]


	def name_search(self, cr, uid, name, args=[], operator='ilike', context={}, limit=80):
        	args2 = args[:]
        	if name:
            		args += [('name', operator, name)]
            		args2 += [('code', operator, name)]
        	ids = self.search(cr, uid, args, limit=limit)
        	ids += self.search(cr, uid, args2, limit=limit)
        	res = self.name_get(cr, uid, ids, context)
        	return res

pathology ()


class medicament (osv.osv):

	def name_get(self, cr, uid, ids, context={}):
		if not len(ids):
			return []
		rec_name = 'name'
		res = [(r['id'], r[rec_name][1]) for r in self.read(cr, uid, ids, [rec_name], context)]
		return res

	_name = "medical.medicament"
	_columns = {
		'name' : fields.many2one ('product.product','Name', domain=[('is_medicament', '=', "1")],help="Commercial Name"),
		'active_component' : fields.char ('active component', size=128, help="Active Component"),
		'therapeutic_action' : fields.char ('Therapeutic effect', size=128, help="Therapeutic action"),
		'composition' : fields.text ('Composition',help="Components"),
		'indications' : fields.text ('Indication',help="Indications"),
		'dosage' : fields.text ('Dosage Instructions',help="Dosage / Indications"),
		'overdosage' : fields.text ('Overdosage',help="Overdosage"),
		'pregnancy_warning' : fields.boolean ('Pregnancy Warning', help="Check when the drug can not be taken during pregnancy or lactancy"),
		'pregnancy' : fields.text ('Pregnancy and Lactancy',help="Warnings for Pregnant Women"),
		'presentation' : fields.text ('Presentation',help="Packaging"),
		'adverse_reaction' : fields.text ('Adverse Reactions'),
		'storage' : fields.text ('Storage Conditions'),
		'notes' : fields.text ('Extra Info'),
		}

medicament ()


class vaccination (osv.osv):
	_name = "medical.vaccination"
	_columns = {
		'name' : fields.many2one ('product.product','Name', domain=[('is_vaccine', '=', "1")], help="Vaccine Name"),
		'date' : fields.datetime ('Date'),
		'dose' : fields.integer ('Dose Number'),
		'observations' : fields.char ('Observations', size=128),
		}
	_defaults = {
                'dose': lambda *a: 1
		}

vaccination ()

class drugs_recreational (osv.osv):
	_name="medical.drugs_recreational"
	_columns = {
		'name': fields.char ('Name', size=128, help="Name of the drug"),
		'street_name': fields.char ('Street names', size=256, help="Common name of the drug in street jargon"),
		'toxicity' : fields.selection([
                                ('0','None'),
                                ('1','Low'),
                                ('2','High'),
                                ('3','Extreme'),
                                ], 'Toxicity', select=True),		
		'addiction_level' : fields.selection([
                                ('0','None'),
                                ('1','Low'),
                                ('2','High'),
                                ('3','Extreme'),
                                ], 'Addiction Level', select=True),		
		'legal_status' : fields.selection([
                                ('0','Legal'),
                                ('1','Illegal'),
                                ], 'Legal Status', select=True),

		'info' : fields.text ('Extra Info'),
		}

drugs_recreational ()

class operational_area (osv.osv):
	_name = "medical.operational_area"
	_columns = {
		'name' :fields.char ('Name', size=128, help="Operational Area of the city or region"),
		'info' :fields.text ('Extra Information'),
		}

        _sql_constraints = [
                ('code_uniq', 'unique (name)', 'The Operational Area code name must be unique')]

operational_area ()

class operational_sector (osv.osv):
	_name = "medical.operational_sector"
	_columns = {
		'name' :fields.char ('Name', size=128, help="Region included in an operational area"),
		'operational_area' :fields.many2one ('medical.operational_area','Operational Area'),
		'info' :fields.text ('Extra Information'),
		}

        _sql_constraints = [
                ('code_uniq', 'unique (name,operational_area)', 'The Operational Sector code and OP Area combination must be unique')]

operational_sector ()

class family_code (osv.osv):
	_name = "medical.family_code"
	_columns = {
		'name' :fields.char ('Name', size=128, help="Family code within an operational sector"),
		'operational_sector' :fields.many2one ('medical.operational_sector','Operational Sector'),
		'members_ids' : fields.many2many ('res.partner', 'family_members_rel','family_id','members_id', 'Members'),
		'info' :fields.text ('Extra Information'),
		}

        _sql_constraints = [
                ('code_uniq', 'unique (name)', 'The Family code name must be unique')]

family_code ()

class speciality (osv.osv):
	_name = "medical.speciality"
	_columns = {
		'name' :fields.char ('Description', size=128, help="ie, Addiction Psychiatry"),
		'code' : fields.char ('Code', size=128, help="ie, ADP"),
	}
        _sql_constraints = [
                ('code_uniq', 'unique (name)', 'The Medical Speciality code must be unique')]

speciality ()


class physician (osv.osv):


	_name = "medical.physician"
	_description = "Information about the doctor"
	_columns = {
		'name' : fields.many2one ('res.partner','Doctor', domain=[('is_doctor', '=', "1")], help="Doctor Name"),
		'institution' : fields.many2one ('res.partner','Institution',domain=[('is_institution', '=', "1")],help="Instituion where she/he works"),
		'code' : fields.char ('ID', size=128, help="MD License ID"),
		'speciality' : fields.many2one ('medical.speciality','Speciality', help="Speciality Code"),
		'info' : fields.text ('Extra info'),
		}

	def name_get(self, cr, uid, ids, context={}):
		if not len(ids):
			return []
		rec_name = 'name'
		res = [(r['id'], r[rec_name][1]) for r in self.read(cr, uid, ids, [rec_name], context)]
		return res

physician ()


class appointment (osv.osv):
	_name = "medical.appointment"


	def name_get(self, cr, uid, ids, context={}):
		if not len(ids):
			return []
		res = []
		for r in self.read(cr, uid, ids, ['rec_name','appointment_date'],context):
			date = str(r['appointment_date'] or '')
			res.append((r['id'], date))
		return res

	def copy(self, cr, uid, id, default=None, context={}):
		default.update({'validity_status':'tobe'})
		return super(appointment,self).copy(cr, uid, id, default, context)
            
	def onchange_appointment_date(self, cr, uid, ids, apt_date):
		if apt_date:
			validity_date = datetime.datetime.fromtimestamp(time.mktime(time.strptime(apt_date,"%Y-%m-%d %H:%M:%S")))
			validity_date = validity_date+datetime.timedelta(days=7)
			v = {'appointment_validity_date':str(validity_date)}
			return {'value': v}
		return {}

	
	_columns = {
        
		'doctor' : fields.many2one ('res.partner','Doctor', domain=[('is_doctor', '=', "1")], required=True, help="Doctor Name"),
		'name' : fields.char ('ID',size=64, readonly=True, required=True),
		'patient' : fields.many2one ('res.partner','Patient', domain=[('is_patient', '=', "1")], help="Patient Name"),
		'appointment_date' : fields.datetime ('Date and Time'),
		'institution' : fields.many2one ('res.partner','Institution', domain=[('is_institution', '=', "1")],help="Medical Center"),
		'doctor' : fields.many2one ('res.partner','Doctor', domain=[('is_doctor', '=', "1")], help="Doctor Name"),
		'speciality' : fields.many2one ('medical.speciality','Speciality', help="Medical Speciality / Sector"),
		'urgency' : fields.selection([
				('a','Normal'),
				('b','Urgent'),
				('c','Medical Emergency'),
				], 'Urgency Level'),

		'comments' : fields.text ('Comments'),

#Additions from Husen Daudi (hda)
		'user_id':fields.related('doctor','user_id',type='many2one',relation='res.partner',string='Doctor'),
		'appointment_validity_date' : fields.datetime ('Validity Date'),
		'validity_status' : fields.selection([('invoiced','Invoiced'),('tobe','To be Invoiced'),('no','No Invoice')],'Status'),
#End of additions from hda

		}
	_order = "appointment_date desc"

	_defaults = {
                'urgency': lambda *a: 'a',
	        'name': lambda self, cr, uid, context=None: \
		self.pool.get('ir.sequence').get(cr, uid, 'medical.appointment'),
		'appointment_date': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),
		'validity_status': lambda *a: 'tobe',
        	}
	

appointment ()



class ethnic_group (osv.osv):
	_name ="medical.ethnicity"
	_columns = {
		'name' : fields.char ('Ethnic group',size=128),
		'code' : fields.char ('Code',size=64),
		}

ethnic_group ()

class occupation (osv.osv):
	_name = "medical.occupation"
	_description = "Occupation / Job"
	_columns = {
		'name' : fields.char ('Occupation', size=128),
		'code' : fields.char ('Code', size=64),
		}
occupation ()

class medical_dose (osv.osv):
	_name = "medical.dose.unit"
	_columns = {
		'name' : fields.char ('Unit',size=32),
		'desc' : fields.char ('Description',size=64),
		}

medical_dose ()


class medical_drug_route (osv.osv):
	_name = "medical.drug.route"
	_columns = {
		'name' : fields.char ('Route',size=64, translate=True),
		'code' : fields.char ('Code',size=32),
		}
medical_drug_route ()


class patient_disease_info (osv.osv):
	_name = "medical.patient.disease"
	_description = "Disease info"
	_columns = {
		'name' : fields.many2one ('medical.pathology','Medical Condition',required=True, help="Disease"),
		'doctor' : fields.many2one('medical.physician','Doctor', help="Doctor who treated or diagnosed the patient"),
		'diagnosed_date': fields.date ('Date of Diagnosis'),
		'age': fields.char ('Age when diagnosed',size=3,help='Patient age at the moment of the diagnosis. Can be estimative'),
		'pregnancy_warning': fields.boolean ('Pregnancy warning'),
		'weeks_of_pregnancy' : fields.integer ('Contracted in pregnancy week #'),
		'description' : fields.char ('Description', size=128),
		'is_on_treatment' : fields.boolean ('Currently on Treatment'),
		'pcs_code' : fields.many2one ('medical.procedure','Code', help="Procedure code, for example, ICD-10-PCS Code 7-character string"),
		'treatment_description' : fields.char ('Treatment Description',size=128),
		'date_start_treatment' : fields.date ('Start of treatment'),
		'date_stop_treatment' : fields.date ('End of treatment'),
		'status' : fields.selection ([
			('c','chronic'),
			('s','status quo'),
			('h','healed'),
			('i','improving'),
			('w','worsening'),
			], 'Status of the disease', select=True),
		'extra_info' : fields.text ('Extra Info'),
		}

patient_disease_info ()





	
class patient_data (osv.osv):

	def name_get(self, cr, user, ids, context={}):
		if not len(ids):
			return []
		def _name_get(d):
			name = d.get('name','')
			id = d.get('patient_id',False)
			if id:
				name = '[%s] %s' % (id,name[1])
			return (d['id'], name)
		result = map(_name_get, self.read(cr, user, ids, ['name','patient_id'], context))
		return result

	def name_search(self, cr, user, name='', args=None, operator='ilike', context=None, limit=80):
		if not args:
			args=[]
		if not context:
			context={}
		if name:
			ids = self.search(cr, user, [('patient_id','=',name)]+ args, limit=limit, context=context)
			if not len(ids):
				ids += self.search(cr, user, [('name',operator,name)]+ args, limit=limit, context=context)
		else:
			ids = self.search(cr, user, args, limit=limit, context=context)
		result = self.name_get(cr, user, ids, context)
		return result    


# Automatically assign the family code

	def onchange_partnerid (self, cr, uid, ids, partner):
		family_code_id = ""
		if partner:
 			cr.execute ('select family_id from family_members_rel where members_id=%s limit 1',(partner,))
			try:
				family_code_id = str(cr.fetchone()[0])
			except:
				family_code_id = ""
			
		v = {'family_code':family_code_id}
		
		return {'value': v}	

	
# Get the patient age in the following format : "YEARS MONTHS DAYS"
		
	def _patient_age(self, cr, uid, ids, name, arg, context={}):
		def compute_age_from_dates (patient_dob):
			now=DateTime.now()
			if (patient_dob):
				dob=DateTime.strptime(patient_dob,'%Y-%m-%d')
				delta=DateTime.Age (now, dob)
				years_months_days = str(delta.years) +" "+ str(delta.months) +" "+" "+ str(delta.days)
			else:
				years_months_days = "No DoB !"
			
 
			return years_months_days
		result={}
	        for patient_data in self.browse(cr, uid, ids, context=context):
	            result[patient_data.id] = compute_age_from_dates (patient_data.dob)
	        return result

	_name = "medical.patient"
	_description = "Patient related information"
	_columns = {
                'name' : fields.many2one('res.partner','Patient', required="1", domain=[('is_patient', '=', True)], help="Patient Name"),
                'patient_id': fields.char('ID', size=64, required=True, select=True),	
		'lastname' : fields.related ('name','lastname',type='char',string='Lastname'), 
		'family_code' : fields.many2one ('medical.family_code','Family',help="Family Code"),
		'identifier' : fields.related ('name','ref',type='char',string='ID'),
		'photo' : fields.binary ('Picture'),
		'dob' : fields.date ('Date of Birth'),
		'age' : fields.function(_patient_age, method=True, type='char', string='Patient Age'),
		'sex' : fields.selection([
                                ('m','Male'),
                                ('f','Female'),
                                ], 'Sex', select=True),
		'marital_status' : fields.selection([
                                ('s','Single'),
                                ('m','Married'),
				('w','Widowed'),
				('d','Divorced'),
				('x','Separated'),
                                ], 'Marital Status'),
		'blood_type' : fields.selection([
				('A','A'),
				('B','B'),
				('AB','AB'),
				('O','O'),
				], 'Blood Type'),
		'rh' : fields.selection([
				('+','+'),
				('-','-'),
				], 'Rh'),



		'ethnic_group' : fields.many2one ('medical.ethnicity','Ethnic group'),
		'allergy_info' : fields.text ('Allergy Info'),
		'allergies': fields.many2many ('medical.pathology','patient_medical_allergies_rel','patient_id','allergy_id',"Allergies"),
		'vaccinations': fields.many2many ('medical.vaccination','patient_medical_vaccination','patient_id','vaccination_id',"Vaccinations"),
		'medications' : fields.one2many('medical.patient.medication','name','Medications'),
		'disease' : fields.many2many ('medical.patient.disease', 'patient_disease_rel','patient_id','disease_id', 'Diseases'),
		'evaluation_ids' : fields.one2many ('medical.patient.evaluation','name','Evaluation'),
		'admissions_ids' : fields.one2many ('medical.patient.admission','name','Admission / Discharge'),
		'notes' : fields.text ('Notes'),
		'deceased' : fields.boolean ('Deceased',help="Mark if the patient has died"),
		'dod' : fields.datetime ('Date of Death'),
		'cod' : fields.many2one ('medical.pathology', 'Cause of Death'),

	}

	_defaults={
		'patient_id': lambda obj, cr, uid, context: obj.pool.get('ir.sequence').get(cr, uid, 'medical.patient'),
		}
		
        _sql_constraints = [
                ('name_uniq', 'unique (name)', 'The Patient already exists')]

patient_data ()

class patient_medication (osv.osv):

#Additions from Husen Daudi (hda)

#	def _get_default_dr(self, cr, uid, context={}):
#		partner_id = self.pool.get('res.partner').search(cr,uid,[('user_id','=',uid)])
#		if partner_id:
#			dr_id = self.pool.get('medical.physician').search(cr,uid,[('name','=',partner_id[0])])
#			if dr_id:
#				return dr_id[0]
#			else:
#				raise osv.except_osv(_('Error !'),
#				_('There is no physician defined ' \
#				'for current user.'))
#		else:
#			return False

#	def search(self, cr, user, args, offset=0, limit=None, order=None, context=None, count=False):
#		if context is None:
#			context = {}
#		if args and args[0] and args[0][0]=='doctor':
#			return super(patient_medication,self).search(cr, user, args, offset, limit, order, context=context, count=count)
#		dr_id = self._get_default_dr(cr, user, context)
#		if dr_id:
#			args.append(('doctor','=',dr_id))
#			return super(patient_medication,self).search(cr, user, args, offset, limit, order, context=context, count=count)
#		else:
#			return []

#End of addittions from hda



	_name = "medical.patient.medication"
	_columns = {
		'name' : fields.many2one ('medical.patient','Patient ID',readonly=True),
		'medicament' : fields.many2one ('medical.medicament','Medicament',help="Prescribed Medicament"),
		'prescription_id' : fields.char ('Prescription ID', size=64, help='Prescription ID. It is useful to group different medicaments in a single prescription'),
		'indication' : fields.many2one ('medical.pathology','Indication'),
		'dose' : fields.integer ('Dose',help="Amount of medication (ie, 250 mg ) each time the patient takes it"),
		'dose_unit' : fields.many2one ('medical.dose.unit','dose unit', help="Unit of measure for the medication to be taken"),
		'route' : fields.many2one ('medical.drug.route','Administration Route',help="HL7 drug administration route code."),
		'frequency' : fields.integer ('Frequency', help="Time in between doses the patient must wait (ie, for 1 pill each 8 hours, put here 8 and select 'hours' in the unit field"),
		'frequency_unit' : fields.selection ([
			('s','seconds'),
			('m','minutes'),
			('h','hours'),
			('d','days'),
			('w','weeks'),
			('wr','when required'),
			], 'unit', select=True),
		'duration' : fields.integer ('Treatment duration',help="Period that the patient must take the medication. in minutes, hours, days, months, years or indefinately"),
		'duration_period' : fields.selection([
                                ('m','minutes'),
                                ('h','hours'),
				('d','days'),
				('m','months'),
				('y','years'),
				('i','indefinate'),
                               ], 'Period',help="Period that the patient must take the medication. in minutes, hours, days, months, years or indefinately", select=True),

		'review' : fields.datetime ('Review'),
		'quantity' : fields.integer ('Quantity'),
		'refills' : fields.integer ('Refills #'),
		'allow_substitution' : fields.boolean('Allow substituion'),  
		'start_treatment' : fields.datetime ('Start of treatment'),
		'end_treatment' : fields.datetime ('End of treatment'),
		'discontinued' :  fields.boolean('Discontinued'),
		'course_completed' : fields.boolean('Course Completed'),
		'doctor' : fields.many2one('medical.physician','Doctor', help="Doctor who prescribed the medicament"),
		'active' : fields.boolean('Active',help="Check this option if the patient is currently taking the medication"),
		'discontinued_reason' : fields.char ('Reason for discontinuation', size=128, help="Short description for discontinuing the treatment"),
		'adverse_reaction' : fields.text ('Adverse Reactions',help="Specific side effects or adverse reactions that the patient experienced"),
		'notes' : fields.text ('Extra Info'),
		'patient_id' : fields.many2one('medical.patient','Patient'),		
		}
	_defaults = {
                'frequency_unit': lambda *a: 'h',
		'start_treatment': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),
		'prescription_id': lambda *a : '/',
#		'doctor': _get_default_dr,
		'active': lambda *a : True,
                'quantity': lambda *a: 1
                }


	
patient_medication ()

class patient_evaluation (osv.osv):

	_name = "medical.patient.evaluation"
	_description = "evaluation"
	_columns = {
		'name' : fields.many2one ('medical.patient','Patient ID', readonly=True),
                'evaluation_date' : fields.many2one ('medical.appointment','Evaluation Date'),
		'evaluation_endtime' : fields.datetime ('End of Evaluation'),
		'next_evaluation' : fields.many2one ('medical.appointment','Next evaluation'),
		'user_id' : fields.many2one ('res.users','Doctor', readonly=True),
		'derived_from' : fields.many2one('medical.physician','Derived from Doctor', help="Physician who escalated / derived the case"), 
		'derived_to' : fields.many2one('medical.physician','Derived to Doctor', help="Physician to whom escalate / derive the case"), 
		'evaluation_type' : fields.selection([
                                ('a','Ambulatory'),
                                ('e','Emergengy'),
                                ('i','Hospitalary'),
                                ('pa','Pre-arraganged appointment'),
                                ('pc','Periodic control'),
			        ('p','Phone call'),
                                ], 'Evaluation Type', select=True),
		'signs' : fields.text ('Signs'),
		'symptoms' : fields.text ('Symptoms'),
		'chief_complaint' : fields.char ('Chief Complaint', size=128,help='Chief Complaint'),
		'notes_complaint' : fields.text ('Complaint details'),
		'glycemia' : fields.float('Glycemia'),
		'cholesterol_total' : fields.float ('Cholesterol'),
		'hdl' : fields.integer ('HDL'),
		'ldl' : fields.integer ('LDL'),
		'tag' : fields.integer ('TAGs',help="Triacylglycerols (triglicerides) level"),
		'systolic' : fields.integer('Systolic Pressure'),
		'diastolic' : fields.integer('Diastolic Pressure'),
		'bpm' : fields.integer ('Heart Rate',help="Heart rated expressed in beats per minute"),
		'temperature' : fields.float('temperature (celsius)'),
		'weight' : fields.float('weight (kg)'),
		'height' : fields.float('height (cm)'),
		'bmi' : fields.float('Body Mass Index'),
		'abdominal_circ' : fields.float('Abdominal Circumference'),
		'loc' : fields.integer('Level of Consciousness', help="Level of Consciousness - on Glasgow Coma Scale :  1=coma - 15=normal"),
		'loc_eyes' : fields.integer('Level of Consciousness - Eyes', help="Eyes Response - Glasgow Coma Scale - 1 to 4"),
		'loc_verbal' : fields.integer('Level of Consciousness - Verbal', help="Eyes Response - Glasgow Coma Scale - 1 to 5"),
		'loc_motor' : fields.integer('Level of Consciousness - Motor', help="Eyes Response - Glasgow Coma Scale - 1 to 6"),		
		'mood' : fields.integer ('Mood',help="1=very angry - 5=very happy"),
		'diagnosis' : fields.many2one ('medical.pathology','Presumptive Diagnosis', help="Presumptive Diagnosis"),
		'info_diagnosis' : fields.text('Presumptive Diagnosis: Extra Info'),
		'directions' : fields.text('Medical Directions'),
		'actions' : fields.many2many('medical.procedure', 'patient_procedures_rel','patient_id','procedure_ids','Procedures'),
		'symptom_thirst' : fields.boolean ('Frequently thirsty'),
		'symptom_hunger' : fields.boolean ('Frequently hungry'),
		'symptom_urination' : fields.boolean ('Frequent urination'),
		'symptom_chest_pain' : fields.boolean ('Chest Pain'),
		'symptom_chest_pain_excercise' : fields.boolean ('Chest Pain on excercise only'),
		'symptom_stress' : fields.boolean ('Stressed-out'),
		'symptom_weight_change' : fields.boolean ('Sudden weight change'),
		'symptom_bruise' : fields.boolean ('Easily Bruised or bleeding'),
		'symptom_fatigue' : fields.boolean ('Feels tired'),
		'symptom_dyspnea' : fields.boolean ('Difficulty breathing'),
		'symptom_swollen_limbs' : fields.boolean ('Swollen Limbs'),
		'symptom_headache' : fields.boolean ('Frequent Headaches'),
		'symptom_memory_loss' : fields.boolean ('Memory loss'),
		'symptom_fainting' : fields.boolean ('Fainting Episode'),
		'symptom_blurry_vision' : fields.boolean ('Blurry vision'),
		'symptom_mood_swings' : fields.boolean ('Mood swings'),
		
		'notes' : fields.text ('Notes'),
	}

	_defaults = {
                'loc_eyes': lambda *a: 4,
                'loc_verbal': lambda *a: 5,
                'loc_motor': lambda *a: 6,
		'evaluation_type': lambda *a: 'pa',
		'user_id': lambda obj, cr, uid, context: uid
        }


	def onchange_height_weight (self, cr, uid, ids, height, weight):
		if height:
			v = {'bmi':weight/((height/100)**2)}
		else:
			v = {'bmi':0}

		return {'value': v}	
				
	def onchange_loc (self, cr, uid, ids, loc_motor, loc_eyes, loc_verbal):
		v = {'loc':loc_motor + loc_eyes + loc_verbal}
		return {'value': v}	


patient_evaluation ()

# HEALTH CENTER / HOSPITAL INFRASTRUCTURE

class hospital_building (osv.osv):
	_name = "medical.hospital.building"
	_columns = {
		'name' : fields.char ('Name', size=128, help="Name of the building within the institution"),
		'institution' : fields.many2one ('res.partner','Institution', domain=[('is_institution', '=', "1")],help="Medical Center"),
		'code' : fields.char ('Code', size=64),
		'extra_info' : fields.text ('Extra Info'),
		}
hospital_building ()

class hospital_unit (osv.osv):
	_name = "medical.hospital.unit"
	_columns = {
		'name' : fields.char ('Name', size=128, help="Name of the unit, eg Neonatal, Intensive Care, ..."),
		'institution' : fields.many2one ('res.partner','Institution', domain=[('is_institution', '=', "1")],help="Medical Center"),
		'code' : fields.char ('Code', size=64),
		'extra_info' : fields.text ('Extra Info'),
		}
hospital_unit ()

class hospital_ward (osv.osv):
	_name = "medical.hospital.ward"
	_columns = {
		'name' : fields.char ('Name', size=128, help="Ward code"),
		'institution' : fields.many2one ('res.partner','Institution', domain=[('is_institution', '=', "1")],help="Medical Center"),
		'building' : fields.many2one ('medical.hospital.building','Building'),
		'floor' : fields.integer ('Floor Number'),
		'unit' : fields.many2one ('medical.hospital.unit','Unit'),
		'private' : fields.boolean ('Private',help="Check this option for private wards"),
		'number_of_beds' : fields.integer ('Number of beds',help="Number of patients per ward"),
		'telephone' : fields.boolean ('Telephone access'),
		'ac' : fields.boolean ('Air Conditioning'),
		'private_bathroom' : fields.boolean ('Private Bathroom'),
		'guest_sofa' : fields.boolean ('Guest sofa-bed'),
		'tv' : fields.boolean ('Television'),
		'internet' : fields.boolean ('Internet Access'),
		'refrigerator' : fields.boolean ('Refrigetator'),
		'microwave' : fields.boolean ('Microwave'),
		'gender' : fields.selection ((('men','Men Ward'),('women','Women Ward'),('unisex','Unisex')),'Gender', required=True),
		'state': fields.selection((('beds_available','Beds available'),('full','Full'),('bio_hazard','Biological Hazard'),('na','Not available')),'Status', required=True),
		'extra_info' : fields.text ('Extra Info'),
		}

	_defaults = {
                'number_of_beds': lambda *a: 1,
		}

hospital_ward ()

class hospital_bed (osv.osv):
	_name = "medical.hospital.bed"
	_columns = {
		'name' : fields.char ('Number',size=128, help="Bed Number / Code"),
		'category' : fields.many2one ('product.product','Category', domain=[('is_bed', '=', "1")], help="Bed type"),
		'ward' : fields.many2one ('medical.hospital.ward','Ward'),
		'bed_type' : fields.selection((('gatch','Gatch Bed'),('electric','Electric'),('stretcher','Stretcher'),('low','Low Bed'),('low_air_loss','Low Air Loss'),('circo_electric','Circo Electric'),('clinitron','Clinitron')),'Bed Type', required=True),
		'telephone_number' : fields.char ('Telephone Number',size=128, help="Telephone number / Extension"),
		'extra_info' : fields.text ('Extra Info'),
		'state': fields.selection((('free','Free'),('reserved','Reserved'),('occupied','Occupied'),('na','Not available')),'Status', required=True),
		}

	_defaults = {
                'bed_type': lambda *a: 'gatch',
		}

hospital_bed ()


class patient_admission (osv.osv):
	_name = "medical.patient.admission"
	_columns = {
		'name' : fields.many2one ('medical.patient','Patient ID',readonly=True),
		'admission_type' : fields.selection([
                                ('in','Inpatient'),
                                ('out','Outpatient'),
                                ], 'Inpatient/Outpatient', help="Inpatient = The patient requires at least one night at the hospital.\nOutpatient = Ambulatory patient. Won't stay overnight.",select=True),
		'date_admission' : fields.date ('Admission date'),
		'date_discharge' : fields.date ('Discharge date'),
		'attending_physician' : fields.many2one ('medical.physician','Attending Physician'),
		'operating_physician' : fields.many2one ('medical.physician','Operating Physician'),
		'admission_reason' : fields.many2one ('medical.pathology','Reason for Admission', help="Reason for Admission"),
		'bed' : fields.many2one ('medical.hospital.bed','Hospital Bed'),
		'nursing_plan' : fields.text ('Nursing Plan'),
		'discharge_plan' : fields.text ('Discharge Plan'),
		'extra_info' : fields.text ('Extra Info'),
		}

	_defaults = {
                'admission_type': lambda *a: 'in',
		}
patient_admission ()



